<?php
/*
    Description:
        1. Verify an incoming Stripe event before it is processed further
        2. For successful payments, the transaction details are checked before they are added in WHMCS for an invoice
        3. The customer will also receive an email to inform them whether their payment was successful or had failed for an invoice

    References:
        1. https://stripe.com/docs/webhooks/signatures#verify-official-libraries
        2. https://github.com/WHMCS/sample-gateway-module/blob/master/modules/gateways/callback/gatewaymodule.php
        3. https://developers.whmcs.com/payment-gateways/callbacks/
        4. https://developers.whmcs.com/api-reference/addinvoicepayment/
        5. https://developers.whmcs.com/api-reference/sendemail/

    Last Updated: 27-FEB-21
*/

namespace StripeFPX;

use \Stripe\Webhook;
use \StripeFPX\Utility;

class Callback
{
    private $module;

    public function __construct()
    {
        $this->module = getGatewayVariables('stripe_fpx');  // Retrieve module parameters that will be used in this class
    }

    public function process()
    {
        http_response_code(403);    // By default, send a 403 error response to Stripe to indicate that there was a problem processing the incoming event

        $event = $this->verifyStripeEvent();    // Firstly, verify if the incoming Stripe event is valid

        $payment_intent = $event['data']['object'];     // Retrieve the payment intent after a Stripe event has been verified

        // Process the payment intent only if it was successful or had failed. Other events are ignored for the time being
        if ( Utility::filterID($payment_intent['id'], 'pi_') ) {

            if ( $event['type'] === 'payment_intent.succeeded' ) {

                $this->paymentSuccess($payment_intent);

            } elseif ( $event['type'] === 'payment_intent.payment_failed' ) {

                $this->paymentFailed($payment_intent);

            }

        }

        http_response_code(200);    // If there was no problem in processing the event, send a successful 200 response to Stripe to confirm receipt of an event
    }

    private function verifyStripeEvent()
    {
        $key = $this->getWebhookKey();
        $signature = $_SERVER['HTTP_STRIPE_SIGNATURE'];
        $payload = file_get_contents('php://input');

        logModuleCall( $this->module['name'], __METHOD__, NULL, array($key, $signature, $payload) );

        return Webhook::constructEvent($payload, $signature, $key);     // This will throw an exception if the Stripe event has an invalid payload or signature
    }

    private function getWebhookKey()
    {
        if ( empty( $this->module['webhook_key'] ) ) {
            throw new \Exception('Please configure the webhook key');
        }

        return $this->module['webhook_key'];
    }

    private function paymentSuccess($payment_intent)
    {
        logTransaction($this->module['name'], $payment_intent, 'SUCCESS');

        // Retrieve the charge object from the payment intent
        $charge = $payment_intent['charges']['data'];
        $charge = $charge[0];

        if ( $charge['status'] === 'succeeded' ) {
            // Check if invoice ID from the charge object is a valid invoice number in WHMCS
            $invoice_id = $charge['metadata']['whmcs_invoice_id'];
            $invoice_id = checkCbInvoiceID( $invoice_id, $this->module['name'] );

            // Check if transaction ID from the charge object is unique to ensure there are no duplicates being recorded in WHMCS
            $transaction_id = $charge['payment_method_details']['fpx']['transaction_id'];
            checkCbTransID($transaction_id);

            $amount = $charge['amount'] / 100;  // convert to decimal format

            // After the invoice ID and transaction ID are checked, add the transaction details in WHMCS
            addInvoicePayment($invoice_id, $transaction_id, $amount, 0.00, $this->module['paymentmethod'], true);

            // Retrieve relevant parameters from the payment intent to be used for sending an email
            $date_time = $charge['created'];
            $bank_name = $charge['payment_method_details']['fpx']['bank'];

            // Construct the email parameters for a successful payment
            $email = array();
            $email['subject'] = 'Invoice Payment Confirmation';
            $email['template'] = file_get_contents(__DIR__ . '/../../../stripe_fpx/templates/emails/payment_success.tpl');
            $email['placeholders']['date_time'] = Utility::formatUnixTime($date_time);
            $email['placeholders']['bank_name'] = Utility::sanitizeBankName($bank_name);

            $this->sendEmail($invoice_id, $email);
        }
    }

    private function paymentFailed($payment_intent)
    {
        logTransaction($this->module['name'], $payment_intent, 'FAILED');

        // Retrieve relevant parameters from the payment intent to be used for sending an email
        $invoice_id = $payment_intent['metadata']['whmcs_invoice_id'];
        $date_time = $payment_intent['last_payment_error']['payment_method']['created'];
        $bank_name = $payment_intent['last_payment_error']['payment_method']['fpx']['bank'];

        // Construct the email parameters for a failed payment
        $email = array();
        $email['subject'] = 'FPX Payment Failed';
        $email['template'] = file_get_contents(__DIR__ . '/../../../stripe_fpx/templates/emails/payment_failed.tpl');
        $email['placeholders']['date_time'] = Utility::formatUnixTime($date_time);
        $email['placeholders']['bank_name'] = Utility::sanitizeBankName($bank_name);

        $this->sendEmail($invoice_id, $email);
    }

    private function sendEmail($invoice_id, $email)
    {
        $parameters = array();
        $parameters['id'] = $invoice_id;
        $parameters['customtype'] = 'invoice';
        $parameters['custommessage'] = str_replace( PHP_EOL, '', $email['template'] );      // this removes extra line breaks set by WHMCS in the email template
        $parameters['customsubject'] = $email['subject'];
        $parameters['customvars'] = $email['placeholders'];

        localAPI('SendEmail', $parameters);     // This sends an email using WHMCS's API
    }
}