<?php
/*
    Description:
        1. Create a new payment intent in Stripe whenever the client views an invoice
        2. Checks if a customer record has already been created in Stripe. Otherwise, a new customer record will be created
        3. Generate the Stripe <div> container to be displayed on the invoice

    References:
        1. https://stripe.com/docs/api/payment_intents/create?lang=php
        2. https://stripe.com/docs/api/customers/create?lang=php
        3. https://stripe.com/docs/api/customers/list?lang=php

    Last Updated: 3-JUL-21
*/

namespace StripeFPX;

use \Stripe\StripeClient;
use \StripeFPX\Utility;

class PaymentIntent
{
    private $module;
    private $client;
    private $invoice;
    private $stripe;

    public function __construct($whmcs)
    {
        // Do not proceed if API keys are not set in the module
        if ( ! Utility::getApiKeys($whmcs) ) {
            throw new \Exception('Please configure the API keys');
        }

        // Parse WHMCS parameters passed to the _link function and assign an array containing relevant values that will be used in this class
        $this->module = Utility::moduleParser($whmcs);
        $this->client = Utility::clientParser($whmcs);
        $this->invoice = Utility::invoiceParser($whmcs) ;

        // Create and initialize a Stripe client
        $this->stripe = new StripeClient( $this->module['secret_key'] );
    }

    public function create()
    {
        // Create a new payment intent for an invoice using the parameters below
        $parameters = array();
        $parameters['amount'] = $this->invoice['amount'];
        $parameters['currency'] = 'myr';
        $parameters['customer'] = $this->getOrCreateCustomer();     // This should return a valid customer ID
        $parameters['description'] = 'Invoice #' . $this->invoice['id'] . ' from ' . parse_url( $this->invoice['url'], PHP_URL_HOST );
        $parameters['metadata'] = $this->invoice['metadata'];
        $parameters['payment_method_types'] = ['fpx'];
        $parameters['statement_descriptor'] = 'Invoice #' . $this->invoice['id'];

        // The idempotency key prevents multiple payment intents being created for the same invoice. See https://stripe.com/docs/api/idempotent_requests
        $idempotency_key = md5( serialize($parameters) );
        $idempotency_key = array('idempotency_key' => $idempotency_key);

        $payment_intent = $this->stripe->paymentIntents->create($parameters, $idempotency_key);

        logModuleCall($this->module['name'], __METHOD__, array_merge($parameters, $idempotency_key), $payment_intent);

        // Check if the client secret value is valid (e.g. pi_1DnV6p2eZvKYlo2CLAYLuidr_secret_wvvbEKIwrJwJhZHjDzTPbiAZd)
        if ( Utility::filterID($payment_intent['client_secret'], 'pi_') ) {
            logActivity("Payment intent created in Stripe for Invoice #{$payment_intent->metadata->whmcs_invoice_id}");

            return $this->generateOutput( $payment_intent['client_secret'] );
        }
    }

    private function getOrCreateCustomer()
    {
        // Firstly, check if a customer record has been created using an email address
        $parameters = array( 'email' => $this->client['email'] );

        $customer = $this->stripe->customers->all($parameters);
        $customer = $customer['data'][0];

        // Return the customer ID if a record is found
        if ( Utility::filterID($customer['id'], 'cus_') ) {
            return $customer['id'];
        }

        // If a customer record is not found, create a new record using the parameters below
        $parameters = array();
        $parameters['description'] = $this->client['full_name'] . ' from ' . parse_url( $this->client['url'], PHP_URL_HOST );
        $parameters['email'] = $this->client['email'];
        $parameters['metadata'] = $this->client['metadata'];
        $parameters['name'] = $this->client['full_name'];

        $customer = $this->stripe->customers->create($parameters);

        logModuleCall($this->module['name'], __METHOD__, $parameters, $customer);

        // Check if the customer ID is valid (e.g. cus_IzwRZAqY76hlfk)
        if ( Utility::filterID($customer['id'], 'cus_') ) {
            logActivity("New customer record for {$customer->email} created in Stripe");

            return $customer['id'];
        }

        return NULL;
    }

    private function generateOutput($client_secret)
    {
        // Set new values for all placeholders (%s) in the Stripe <div> container
        $placeholders = array();
        $placeholders['publishable_key'] = $this->module['publishable_key'];
        $placeholders['client_secret'] = $client_secret;
        $placeholders['return_url'] = $this->invoice['url'];
        $placeholders['call_to_action'] = $this->module['call_to_action'];

        $html = file_get_contents(__DIR__ . '/../../../stripe_fpx/templates/payment_link.tpl');
        $html = vsprintf($html, $placeholders);     // This replaces all placeholders (%s) found in the above template

        return $html;
    }
}